﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ZedGraph;
using System.Drawing;
using ListWithChangedEvent;
using System.Windows.Forms;

namespace netvis
{
   class Network
    {
        protected GUI GUinterface;
        protected Parser parser;
        protected DateTime epoch = new DateTime(1900, 1, 1);
        protected static Network holder = null;
        protected Color[] colors = new Color[] {Color.Red,Color.Blue,Color.Green,Color.Violet,Color.Purple,Color.Pink,Color.Black,Color.DarkSeaGreen,Color.SlateGray };
        private Network(GUI gui) {
            GUinterface = gui;
        }
        public static Network instance(GUI gui)
        {
            if (holder == null) {
                holder = new Network(gui);
            }
        return holder;
        }
        public void open()
        {
            FolderBrowserDialog dialog = new FolderBrowserDialog();
            dialog.RootFolder = System.Environment.SpecialFolder.MyComputer;
            dialog.ShowDialog();
            parser = new CSVParser(dialog.SelectedPath);
            parser.getNext(10000);


            int i = 0;
            PointPairList counts = new PointPairList();
            foreach (NetworkInterface nt in parser.getInterfaces())
            {
                counts = new PointPairList();
                GUinterface.addLineToGraph(counts, colors[i], nt.interfaceName);
                i++;
            }
            counts = new PointPairList();
            GUinterface.addLineToGraph(counts, colors[i], "Max");
        }
        public void updateGraph(NetworkInterface selected) {
            PointPairList points = new PointPairList();
            TimeSpan target = new Year(null,1);
            if (GUinterface.comboBox2.SelectedIndex == 0) {
                target = new Month(null, DateTime.Now);
            }
            else if (GUinterface.comboBox2.SelectedIndex == 1) {
                target = new Week(null, DateTime.Now);
            }
            else if (GUinterface.comboBox2.SelectedIndex == 2)
            {
                target = new Day(null, DateTime.Now);
            }
            else if (GUinterface.comboBox2.SelectedIndex == 3)
            {
                target = new Hour(null, DateTime.Now);
            }
            else if (GUinterface.comboBox2.SelectedIndex == 4)
            {
                target = new MinDecade(null, DateTime.Now);
            }
            else if (GUinterface.comboBox2.SelectedIndex == 5)
            {
                target = new Minute(null, DateTime.Now);
            }
            else if (GUinterface.comboBox2.SelectedIndex == 6)
            {
                target = new SecDecade(null, DateTime.Now);
            }
            else if (GUinterface.comboBox2.SelectedIndex == 7)
            {
                target = new Second(null, DateTime.Now);
            }
            //foreach (Node node in (selected).roots) {
            points = selected.getTimeDevs(target);
            //}
            //points = getRec(points,
            points.Sort();
            GUinterface.updateCurveItem(selected.interfaceName, points);

            //NetInterface lagest = selected;
            //Dictionary<NetInterface, ListWithChangedEvent<PacketInfo>> temp2 = parseBuff.getCurrent();
            //foreach (NetInterface nt in temp2.Keys) {
            //    if (temp2[nt].Count>0 && temp2[nt][temp2[nt].Count - 1].getStartTime() - temp2[nt][0].getStartTime() > temp2[lagest][temp2[lagest].Count - 1].getStartTime() - temp2[lagest][0].getStartTime())
            //    {
            //        lagest = nt;
            //    }
            
            //}
            //GUinterface.updateGraph(getMinMax(parseBuff.getCurrent(), lagest));
        }
        public void readMore() {
            parser.getNext(100000);
        }

        //public PointPairList getMinMax(Dictionary<NetInterface, ListWithChangedEvent<PacketInfo>> data, NetInterface selected)
        //{
        //    PointPairList counts = new PointPairList();
        //    ListWithChangedEvent<PacketInfo> pl = parseBuff.getCurrent()[selected];
        //    PacketInfo min = pl.Min();
        //    PacketInfo max = pl.Max();
        //    double gap = ((TimeSpan.FromDays(int.Parse(GUinterface.day.Text)) + new TimeSpan(int.Parse(GUinterface.hour.Text), int.Parse(GUinterface.minute.Text), int.Parse(GUinterface.second.Text))).TotalDays);
        //    double begin = (min.getStartTime() - new DateTime(1900, 1, 1)).TotalDays;
        //    double highest = (max.getStartTime() - new DateTime(1900, 1, 1)).TotalDays;
        //    while (begin < highest)
        //    {
        //        counts.Add(begin, mostInGap(begin, begin += gap, data));
        //    }
        //    return counts;
        //}
        //public int mostInGap(double start, double end, Dictionary<NetInterface, ListWithChangedEvent<Packet>> data)
        //{
        //    int max = 0;
        //    foreach (NetInterface nt in data.Keys)
        //    {
        //        int localMax = 0;
        //        foreach (Packet pck in data[nt])
        //        {
        //            if ((pck.getStartTime() - new DateTime(1900, 1, 1)).TotalDays >= start && (pck.getStartTime() - new DateTime(1900, 1, 1)).TotalDays <= end)
        //            {
        //                localMax++;
        //            }
        //        }
        //        max = localMax > max ? localMax : max;
        //    }
        //    return max;
        //}
        void changeEvent(object sender, EventArgs e)
        {
            Console.WriteLine(e);
            Console.WriteLine(sender);
        }

        public int getTotalPass(){
          int total = 0;
          foreach (NetworkInterface face in parser.getInterfaces())
          {
            total += face.getTotalAllow();
          }
          return total;
      }
        public int getTotalBlock() {
          int total = 0;
          foreach(NetworkInterface face in parser.getInterfaces())
          {
            total += face.getTotalBlock();
          }
          return total;
      }
        public DateTime getStartTime(){
          DateTime ealist = DateTime.MaxValue;
          foreach (NetworkInterface face in parser.getInterfaces())
          {
              DateTime time = face.getEarliestTime();
              if (time < ealist)
              {
                  ealist = time;
              }
          }
          return ealist;
      }
        public DateTime getEndTime() {
          DateTime latest = DateTime.MinValue;
          foreach (NetworkInterface face in parser.getInterfaces())
          {
              DateTime time = face.getLatestTime();
              if (time > latest) {
                  latest = time;
              }
          }
          return latest;
      }
        public List<NetworkInterface> getAllInterfaces() {
          return parser.getInterfaces();
      }

    }
}
